@@ -125,6 +125,7 @@ int modsecurity_init(msc_engine *msce, apr_pool_t *mp) {
         return -1;
     }
 
+#if !defined(MSC_TEST)
 #ifdef __SET_MUTEX_PERMS
 #if AP_SERVER_MAJORVERSION_NUMBER > 1 && AP_SERVER_MINORVERSION_NUMBER > 2
     rc = ap_unixd_set_global_mutex_perms(msce->auditlog_lock);
@@ -153,7 +154,7 @@ int modsecurity_init(msc_engine *msce, apr_pool_t *mp) {
         return -1;
     }
 #endif /* SET_MUTEX_PERMS */
-
+#endif
 
     return 1;
 }
@@ -260,6 +261,12 @@ static apr_status_t modsecurity_tx_cleanup(void *data) {
         msr_log(msr, 1, "%s", my_error_msg);
     }
 
+#if defined(WITH_LUA)
+    #ifdef CACHE_LUA
+    if(msr->L != NULL)  lua_close(msr->L);
+    #endif
+#endif
+
     return APR_SUCCESS;
 }
 
@@ -269,6 +276,7 @@ static apr_status_t modsecurity_tx_cleanup(void *data) {
 apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     const char *s = NULL;
     const apr_array_header_t *arr;
+    char *_cookies = NULL;
     apr_table_entry_t *te;
     int i;
 
@@ -369,6 +377,10 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     msr->pattern_to_sanitize = apr_table_make(msr->mp, 32);
     if (msr->pattern_to_sanitize == NULL) return -1;
 
+    /* remove targets */
+    msr->removed_targets = apr_table_make(msr->mp, 16);
+    if (msr->removed_targets == NULL) return -1;
+
     /* Initialise cookies */
     msr->request_cookies = apr_table_make(msr->mp, 16);
     if (msr->request_cookies == NULL) return -1;
@@ -378,13 +390,24 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     if (msr->matched_vars == NULL) return -1;
     apr_table_clear(msr->matched_vars);
 
+    if(msr->txcfg->max_rule_time > 0)   {
+        msr->perf_rules = apr_table_make(msr->mp, 8);
+        if (msr->perf_rules == NULL) return -1;
+        apr_table_clear(msr->perf_rules);
+    }
+
     /* Locate the cookie headers and parse them */
     arr = apr_table_elts(msr->request_headers);
     te = (apr_table_entry_t *)arr->elts;
     for (i = 0; i < arr->nelts; i++) {
         if (strcasecmp(te[i].key, "Cookie") == 0) {
             if (msr->txcfg->cookie_format == COOKIES_V0) {
-                parse_cookies_v0(msr, te[i].val, msr->request_cookies);
+                _cookies = apr_pstrdup(msr->mp, te[i].val);
+                while((*_cookies != 0)&&(*_cookies != ',')&&(*_cookies != ';')) _cookies++;
+                if(*_cookies == ',')
+                    parse_cookies_v0(msr, te[i].val, msr->request_cookies, ",");
+                else
+                    parse_cookies_v0(msr, te[i].val, msr->request_cookies, ";");
             } else {
                 parse_cookies_v1(msr, te[i].val, msr->request_cookies);
             }
@@ -423,6 +446,9 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     msr->removed_rules_tag = apr_array_make(msr->mp, 16, sizeof(char *));
     if (msr->removed_rules_tag == NULL) return -1;
 
+    msr->removed_rules_msg = apr_array_make(msr->mp, 16, sizeof(char *));
+    if (msr->removed_rules_msg == NULL) return -1;
+
     return 1;
 }
 
