@@ -20,7 +20,7 @@
 #include "msc_util.h"
 #include "msc_parsers.h"
 
-void validate_quotes(modsec_rec *msr, unsigned char *data)  {
+void validate_quotes(modsec_rec *msr, char *data)  {
     int i, len;
 
     if(msr == NULL)
@@ -46,7 +46,6 @@ void validate_quotes(modsec_rec *msr, unsigned char *data)  {
     }
 }
 
-
 #if 0
 static char *multipart_construct_filename(modsec_rec *msr) {
     char c, *p, *q = msr->mpd->mpp->filename;
@@ -123,6 +122,7 @@ static int multipart_parse_content_disposition(modsec_rec *msr, char *c_d_value)
          * technically "'" is invalid and so flag_invalid_quoting is
          * set so the user can deal with it in the rules if they so wish.
          */
+
         if ((*p == '"') || (*p == '\'')) {
             /* quoted */
             char quote = *p;
@@ -273,10 +273,10 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
 
     /* Is this an empty line? */
     if (   ((msr->mpd->buf[0] == '\r')
-          &&(msr->mpd->buf[1] == '\n')
-          &&(msr->mpd->buf[2] == '\0') )
-        || ((msr->mpd->buf[0] == '\n')
-          &&(msr->mpd->buf[1] == '\0') ) )
+                &&(msr->mpd->buf[1] == '\n')
+                &&(msr->mpd->buf[2] == '\0') )
+            || ((msr->mpd->buf[0] == '\n')
+                &&(msr->mpd->buf[1] == '\0') ) )
     { /* Empty line. */
         char *header_value = NULL;
 
@@ -289,7 +289,7 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
         rc = multipart_parse_content_disposition(msr, header_value);
         if (rc < 0) {
             *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid Content-Disposition header (%d): %s.",
-                rc, log_escape_nq(msr->mp, header_value));
+                    rc, log_escape_nq(msr->mp, header_value));
             return -1;
         }
 
@@ -624,6 +624,7 @@ static int multipart_process_boundary(modsec_rec *msr, int last_part, char **err
             &&(msr->mpd->mpp->tmp_file_fd != 0))
         {
             close(msr->mpd->mpp->tmp_file_fd);
+            msr->mpd->mpp->tmp_file_fd = -1;
         }
 
         if (msr->mpd->mpp->type != MULTIPART_FILE) {
@@ -653,6 +654,7 @@ static int multipart_process_boundary(modsec_rec *msr, int last_part, char **err
             }
         }
         else {
+            msr->mpd->flag_invalid_part = 1;
             msr_log(msr, 3, "Multipart: Skipping invalid part %pp (part name missing): "
                 "(offset %u, length %u)", msr->mpd->mpp,
                 msr->mpd->mpp->offset, msr->mpd->mpp->length);
@@ -944,7 +946,7 @@ int multipart_complete(modsec_rec *msr, char **error_msg) {
 
         if (msr->mpd->flag_header_folding) {
             msr_log(msr, 4, "Multipart: Warning: header folding used.");
-        }        
+        }
 
         if (msr->mpd->flag_crlf_line && msr->mpd->flag_lf_line) {
             msr_log(msr, 4, "Multipart: Warning: mixed line endings used (CRLF/LF).");
@@ -961,9 +963,13 @@ int multipart_complete(modsec_rec *msr, char **error_msg) {
             msr_log(msr, 4, "Multipart: Warning: invalid quoting used.");
         }
 
+        if (msr->mpd->flag_invalid_part) {
+            msr_log(msr, 4, "Multipart: Warning: invalid part parsing.");
+        }
+
         if (msr->mpd->flag_invalid_header_folding) {
             msr_log(msr, 4, "Multipart: Warning: invalid header folding used.");
-        }        
+        }
     }
 
     if ((msr->mpd->seen_data != 0) && (msr->mpd->is_complete == 0)) {
