@@ -37,8 +37,8 @@
 #endif
 
 #define MODSEC_VERSION_MAJOR       "2"
-#define MODSEC_VERSION_MINOR       "6"
-#define MODSEC_VERSION_MAINT       "6"
+#define MODSEC_VERSION_MINOR       "7"
+#define MODSEC_VERSION_MAINT       "1"
 #define MODSEC_VERSION_TYPE        ""
 #define MODSEC_VERSION_RELEASE     ""
 
@@ -49,7 +49,19 @@
   MODSEC_VERSION_SUFFIX
 
 /* Apache Module Defines */
+#ifdef	VERSION_IIS
+#define MODSEC_MODULE_NAME "ModSecurity for IIS (Beta)"
+#else
+#ifdef	VERSION_NGINX
+#define MODSEC_MODULE_NAME "ModSecurity for nginx (Beta)"
+#else
+#ifdef	VERSION_STANDALONE
+#define MODSEC_MODULE_NAME "ModSecurity Standalone (Beta)"
+#else
 #define MODSEC_MODULE_NAME "ModSecurity for Apache"
+#endif
+#endif
+#endif
 #define MODSEC_MODULE_VERSION MODSEC_VERSION
 #define MODSEC_MODULE_NAME_FULL MODSEC_MODULE_NAME "/" MODSEC_MODULE_VERSION " (http://www.modsecurity.org/)"
 
