@@ -586,13 +586,13 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
         goto error;
     }
 
-    if(strstr(col_name,"USER") || strstr(col_name,"SESSION"))
+    if(strstr(col_name,"USER") || strstr(col_name,"SESSION") || strstr(col_name, "RESOURCE"))
         dbm_filename = apr_pstrcat(msr->mp, msr->txcfg->data_dir, "/", msr->txcfg->webappid, "_", col_name, NULL);
     else
         dbm_filename = apr_pstrcat(msr->mp, msr->txcfg->data_dir, "/", col_name, NULL);
 
     if (msr->txcfg->debuglog_level >= 9) {
-        msr_log(msr, 1, "collections_remove_stale: Retrieving collection (name \"%s\", filename \"%s\")",log_escape(msr->mp, col_name),
+        msr_log(msr, 9, "collections_remove_stale: Retrieving collection (name \"%s\", filename \"%s\")",log_escape(msr->mp, col_name),
                 log_escape(msr->mp, dbm_filename));
     }
 
